@@ -489,6 +489,7 @@ getPacketTypeAndPayload = do
                        3 -> do len <- remaining
                                bs <- getByteString len
                                return (t, bs)
+                       _ -> error "This should never happen."
         0x40 -> do
                    len <- fmap fromIntegral getPacketLength
                    bs <- getByteString len
@@ -755,7 +756,7 @@ getPubkey DSA = do MPI p <- get
                    MPI q <- get
                    MPI g <- get
                    MPI y <- get
-                   return $ DSAPubKey (D.PublicKey (p, g, q) y)
+                   return $ DSAPubKey (D.PublicKey (D.Params p g q) y)
 getPubkey ElgamalEncryptOnly = getPubkey Elgamal
 getPubkey Elgamal = do MPI p <- get
                        MPI g <- get
@@ -780,7 +781,7 @@ getSecretKey pkp
                pub = (\(RSAPubKey x) -> x) (_pubkey pkp)
            return $ RSAPrivateKey (R.PrivateKey pub d p q dP dQ qinv)
     | _pkalgo pkp == DSA = do MPI x <- get
-                              return $ DSAPrivateKey (D.PrivateKey (0,0,0) x)
+                              return $ DSAPrivateKey (D.PrivateKey (D.Params 0 0 0) x)
     | _pkalgo pkp `elem` [ElgamalEncryptOnly,Elgamal] =
         do MPI x <- get
            return $ ElGamalPrivateKey [x]
@@ -1129,7 +1130,8 @@ getSignaturePayload = do
     pv <- getWord8
     case pv of
         3 -> do
-            hashlen <- getWord8 -- FIXME: must be 5
+            hashlen <- getWord8
+            guard (hashlen == 5)
             st <- getWord8
             ctime <- getWord32be
             eok <- getByteString 8
